# Defining bluealsa PCMs

The bluealsa ALSA pcm plugin communicates directly with the bluealsa service. It is a raw communication without any conversions. To use it, create an ALSA configuration node defining a pcm with type `bluealsa`. [See below for a predefined parameterized pcm, also called "bluealsa", that can avoid the need for writing ALSA configurations.](#the-pre-defined-bluealsa-pcm)

The configuration node has the following fields:

```
pcm.name {
    type bluealsa       # Bluetooth PCM
    device STR          # Device address in format XX:XX:XX:XX:XX:XX
    profile STR         # Profile type (a2dp or sco)
    [delay INT]         # Extra delay (in frames) to be reported to the application (default 0)
    [service STR]       # D-Bus name of bluealsa service (default org.bluealsa)
}
```

The `device` and `profile` fields must be specified so that the plugin can select the correct bluetooth transport; `delay` and `service` are optional.

The value for `device` must be an actual bluetooth MAC address, device names and aliases are not valid.

Use `profile "a2dp"` for A2DP streams, and `profile "sco"` for HSP or HFP streams.

The `delay` option specifies a number of frames to be added to the delay value reported by the plugin to the application. This is necessary because the bluetooth audio profiles (A2DP, HFP, HSP) do not report the delay (latency) from the time a frame is sent to the bluetooth controller to the time it is actually played out by the loudspeaker (or vice-versa for capture streams from microphones). The user must determine this delay by experience; it is only necessary when trying to synchronize the sound, for example with a video or another sound system.

The `service` option specifies the D-Bus name used by the bluealsa service. It defaults to "org.bluealsa", which is the default used by the server, so only needs to be specified in special cases.

A simple, typical, example is to add the following to your `~/.asoundrc` file (where 00:11:22:33:44:55 is the example bluetooth device address and `bt-headset` is the name you have chosen to use for the pcm)
```
pcm.bt-headset {
	type plug
	slave.pcm {
		type bluealsa
		device "00:11:22:33:44:55"
		profile "a2dp"
		delay 12000
	}
}
```

With that definition in your `~/.asoundrc` file, you can then play audio to the headset by sending to device `bt-headset`; for example:
```sh
aplay -D bt-headset music.wav
```

# The pre-defined "bluealsa" pcm

As a convenience for users, the bluez-alsa project also installs a global ALSA configuration file which defines a parameterized pcm device named "bluealsa".

`pcm.bluealsa` is of type plug, so that it automatically converts the audio stream from/to the format required by the bluetooth device. If we compare the `bluealsa` plugin type with the `hw` plugin type, then `pcm.bluealsa` is analogous to `pcm.plughw`. You can see the bluealsa ALSA configuration file here: [20-bluealsa.conf](https://github.com/Arkq/bluez-alsa/blob/master/src/asound/20-bluealsa.conf)

The `pcm.bluealsa` arguments map to the bluealsa plugin parameters as follows:

| plugin parameter | pcm.bluealsa argument name | default |
|-----------|---------------|--|
| device | DEV | defaults.bluealsa.device |
| profile | PROFILE | defaults.bluealsa.profile |
| delay | DELAY | defaults.bluealsa.delay |
| service | SRV | defaults.bluealsa.service |

You can refer to a bluealsa pcm without needing to create a configuration in `~/.asoundrc` with:
```
bluealsa:DEV=00:11:22:33:44:55,PROFILE=a2dp,DELAY=20000,SRV=org.bluealsa
```

The pre-defined configuration also provides values for all the defaults in the above table except `defaults.bluealsa.device`. So the above example could more simply be written as
```
bluealsa:DEV=00:11:22:33:44:55
```

ALSA permits arguments to be given as positional parameters as an alternative to explicitly naming them. **When using positional parameters it is important that the values are given in the correct sequence - DEV,PROFILE,DELAY,SRV**.

> *Note - before bluez-alsa release 3.0.0, the sequence of the arguments was SRV,DEV,PROFILE,DELAY so the following positional parameter examples will not work with releases 2.1.0 and earlier.*

Using positional parameters, we could also write our example as:

```
bluealsa:00:11:22:33:44:55,a2dp,20000,org.bluealsa
```

or, taking advantage of the default definitions again:

```
bluealsa:00:11:22:33:44:55
```

So, without needing to create a `~/asoundrc` file at all, we can send audio to the same headset as in the example above with:
```sh
aplay -D bluealsa:00:11:22:33:44:55 music.wav
```

The defaults can be overridden by defining the ones you want to change in your own configuration (e.g in `~/.asoundrc.conf`) for example:

```
defaults.bluealsa.device "00:11:22:33:44:55"
defaults.bluealsa.profile "sco"
defaults.bluealsa.delay 50
defaults.bluealsa.service "org.bluealsa"
```
